;;; org-src-jump.el --- jump between a source and related Org-mode files

;; Set the `os-org-file' variable in your source file and the
;; `os-src-file' in your Org-mode file as file local variables.

;;; Code:
(require 'imenu)
(require 'which-func)

(defvar os-org-file nil "Org file related to a source buffer.")
(put 'os-org-file 'safe-local-variable 'stringp)

(defvar os-src-file nil "Src file related to an Org-mode buffer.")
(put 'os-src-file 'safe-local-variable 'stringp)

(defun org-src-jump ()
  "Jump between Org-mode and src buffers."
  (interactive)
  (if (equal major-mode 'org-mode)
      ;; jump to function in src file
      (let ((func (org-entry-get (point) "ID")))
        (unless os-src-file (error "Must set `os-src-file' in this buffer."))
        (find-file os-src-file)
        (goto-char (cdr (assoc func imenu--index-alist))))
    ;; jump to heading in org file
    (let* ((func (which-function))
           (m (org-id-find func 'marker)))
      (find-file os-org-file)
      (if m (goto-char m)
        ;; make a new Org-mode heading for this function
        (goto-char (point-max))
        (org-insert-heading t)
        (org-entry-put (point) "ID" func)
        (insert func)
        (org-id-update-id-locations (list os-org-file))))))

(provide 'org-src-jump)
